/*
 * Copyright (C) 2009 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef ScriptEventListener_h
#define ScriptEventListener_h

#include "base/memory/scoped_refptr.h"
#include "bindings/core/v8/ScriptValue.h"
#include "bindings/core/v8/V8LazyEventListener.h"

namespace blink {

class EventListener;
class ExecutionContext;
class LocalFrame;
class Node;
class QualifiedName;
class SourceLocation;

V8LazyEventListener* CreateAttributeEventListener(
    Node*,
    const QualifiedName&,
    const AtomicString& value,
    const AtomicString& event_parameter_name);
V8LazyEventListener* CreateAttributeEventListener(
    LocalFrame*,
    const QualifiedName&,
    const AtomicString& value,
    const AtomicString& event_parameter_name);
v8::Local<v8::Object> EventListenerHandler(ExecutionContext*, EventListener*);
v8::Local<v8::Function> EventListenerEffectiveFunction(
    v8::Isolate*,
    v8::Local<v8::Object> handler);
void GetFunctionLocation(v8::Local<v8::Function>,
                         String& script_id,
                         int& line_number,
                         int& column_number);
std::unique_ptr<SourceLocation> GetFunctionLocation(ExecutionContext*,
                                                    EventListener*);

}  // namespace blink

#endif  // ScriptEventListener_h
